/**
 * jipCam : The Java IP Camera Project
 * Copyright (C) 2005-2006 Jason Thrasher
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package net.sf.jipcam.axis.emulator;

import net.sf.jipcam.axis.CameraAPI;
import net.sf.jipcam.axis.MjpegFifo;
import net.sf.jipcam.axis.MjpegFrame;
import net.sf.jipcam.axis.MjpegFrameParser;
import net.sf.jipcam.axis.MjpegParserListener;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;

import java.util.Enumeration;

import javax.servlet.*;
import javax.servlet.http.*;


/**
 * Servlet to simulate the MJPEG stream coming from the camera.  This allows
 * use of a "canned" stream on the disk.  The stream is "played" in a loop for
 * the HTTP client.  The client should behave as if it's a continuous stream.
 *
 * The CGI request supports modifications to FPS.  This allows the client to run
 * at much higher FPS than the camera actually supports.  The timing of the response
 * MJPEG frames is not very accurate - but good enough up to around 100 FPS on a 2 GHz
 * computer.
 *
 * @author Jason Thrasher
 */
public class MjpegServlet extends HttpServlet {
	private static Logger mLog;
	private static final String CONTENT_TYPE = "multipart/x-mixed-replace; boundary=--myboundary";
	private static int DEFAULT_FPS = 10; //set in web.xml
	private File mMjpegFile; //set in web.xml

	public void init(ServletConfig config) throws ServletException {
		super.init(config);
		mLog = Logger.getLogger(this.getClass());

		String prefix = getServletContext().getRealPath("/");

		//mLog.log(Level.INFO, "context real path = " + prefix);
		String file = config.getInitParameter("mjpeg.file");

		if (file != null) {
			mMjpegFile = new File(prefix + file); //assume relative path

			//check that conf file found
			if (!mMjpegFile.exists()) {
				mMjpegFile = new File(file); //try the absolute path

				if (!mMjpegFile.exists()) {
					mLog.log(Level.FATAL, "mjpeg file not found: " + file);
					throw new ServletException("mjpeg file not found: " + file);
				}
			}
		} else {
			//no config file specified
			mLog.log(Level.FATAL, "mjpeg file not specified");
			throw new ServletException("mjpeg file not specified");
		}

		mLog.log(Level.INFO, "using " + mMjpegFile);

		DEFAULT_FPS = getIntegerParam(config.getInitParameter("mjpeg.fps"),
				DEFAULT_FPS);
	}

	public void doGet(HttpServletRequest request, HttpServletResponse response)
		throws ServletException, IOException {
		//log out the request parameters
		Enumeration names = request.getParameterNames();

		while (names.hasMoreElements()) {
			String name = (String) names.nextElement();
			String value = request.getParameter(name);
			mLog.log(Level.INFO, "request parameter: " + name + "=" + value);
		}

		//try to get the frames per second value from the request		
		int fps = getIntegerParam(request.getParameter("req_fps"), -1);

		if (fps == -1) {
			fps = getIntegerParam(request.getParameter("des_fps"), -1);
		}

		int deltaT = (fps <= 0) ? (1000 / DEFAULT_FPS) : (1000 / fps);

		//send response headers
		response.setContentType(CONTENT_TYPE);
		response.setHeader("Server", "Camd");
		response.setHeader("Connection", "Close");

		OutputStream out = new BufferedOutputStream(response.getOutputStream());

		int byteCount = 0;

		MjpegFifo fifo = new MjpegFifo(DEFAULT_FPS * 10);
		MjpegFifoLoop loop = new MjpegFifoLoop(mMjpegFile, fifo);
		loop.start();

		//wait for the loop to fill the fifo a little
		try {
			Thread.sleep(1000); //startup cost
		} catch (InterruptedException ie) {
			//ignore
		}

		try {
			MjpegFrame frame = null;

			while (loop.isAlive()) {
				frame = fifo.getMjpegFrame();
				out.write(frame.getBytes(), 0, frame.getLength());
				out.flush();

				//calculate the pause between frames
				if (fps <= 0) {
					//use the stream's default, if available
					String deltaS = frame.getProperties().getProperty("Delta-time");

					//					mLog.log(Level.INFO, "Delta-time = " + deltaS);
					deltaT = getIntegerParam(deltaS, deltaT);
				}

				//				mLog.log(Level.INFO, "sleeping ms = " + deltaT);
				Thread.sleep(deltaT);
			}
		} catch (IOException ioe) {
			//usually due to client disconnect
			mLog.log(Level.FATAL, "lost client connection");
		} catch (InterruptedException ie) {
			mLog.log(Level.WARN, ie.getMessage());
		} catch (Exception e) {
			mLog.log(Level.ERROR, e.getMessage());
		} finally {
			loop.stop();
		}

		out.flush();
		out.close();
	}

	public void doPost(HttpServletRequest request, HttpServletResponse response)
		throws ServletException, IOException {
		doGet(request, response);
	}

	/**
	 * Utility method to uniformly parse numbers
	 */
	private int getIntegerParam(String value, int vDefault) {
		int val = vDefault;

		try {
			val = Integer.parseInt(value);
		} catch (Exception e) {
			mLog.log(Level.WARN,
				"failed to parse integer: " + value + " using default: " +
				vDefault);
		}

		return val;
	}
}
